@@ -18,46 +18,52 @@
 #include <errno.h>
 #include <assert.h>
 
-#include <arpa/inet.h>
-#include <sys/socket.h>
-#include <sys/ioctl.h>
-#include <net/if.h>
-#include <linux/if_packet.h>
-
+#include "../lib/includes.h"
 #include "../lib/logger.h"
 #include "../lib/random.h"
 #include "../lib/blacklist.h"
 
-#include "cyclic.h"
-#include "state.h"
+#include "aesrand.h"
+#include "get_gateway.h"
+#include "iterator.h"
 #include "probe_modules/packet.h"
 #include "probe_modules/probe_modules.h"
+#include "shard.h"
+#include "state.h"
 #include "validate.h"
 
+// OS specific functions called by send_run
+static inline int send_packet(int fd, void *buf, int len);
+static inline int send_run_init(int sock);
 
-// lock to manage access to share send state (e.g. counters and cyclic)
-pthread_mutex_t send_mutex = PTHREAD_MUTEX_INITIALIZER;
-// lock to provide thread safety to the user provided send callback
-pthread_mutex_t syncb_mutex = PTHREAD_MUTEX_INITIALIZER;
 
-// globals to handle sending from multiple ip addresses (shared across threads)
-static uint16_t num_src_ports;
-static uint32_t num_addrs;
+// Include the right implementations
+#if defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__)
+#include "send-bsd.h"
+#else /* LINUX */
+#include "send-linux.h"
+#endif /* __APPLE__ || __FreeBSD__ || __NetBSD__ */
+
+// The iterator over the cyclic group
+
+// Lock for send run
+static pthread_mutex_t send_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+// Source IP address for outgoing packets
 static in_addr_t srcip_first;
 static in_addr_t srcip_last;
-// offset send addresses according to a random chosen per scan execution
-// in order to help prevent cross-scan interference
 static uint32_t srcip_offset;
+static uint32_t num_src_addrs;
+
+// Source ports for outgoing packets
+static uint16_t num_src_ports;
 
 // global sender initialize (not thread specific)
-int send_init(void)
+iterator_t* send_init(void)
 {
-	// generate a new primitive root and starting position
-	cyclic_init(0, 0);
-	zsend.first_scanned = cyclic_get_curr_ip();
-
 	// compute number of targets
 	uint64_t allowed = blacklist_count_allowed();
+	assert(allowed <= (1LL << 32));
 	if (allowed == (1LL << 32)) {
 		zsend.targets = 0xFFFFFFFF;
 	} else {
@@ -67,6 +73,10 @@ int send_init(void)
 		zsend.targets = zconf.max_targets;
 	}
 
+	// generate a new primitive root and starting position
+	iterator_t *it;
+	it = iterator_init(zconf.senders, zconf.shard_num, zconf.total_shards);
+
 	// process the dotted-notation addresses passed to ZMAP and determine
 	// the source addresses from which we'll send packets;
 	srcip_first = inet_addr(zconf.source_ip_first);
@@ -79,18 +89,26 @@ int send_init(void)
 		log_fatal("send", "invalid end source ip address: `%s'",
 				zconf.source_ip_last);
 	}
+	log_debug("send", "srcip_first: %u", srcip_first);
+	log_debug("send", "srcip_last: %u", srcip_last);
 	if (srcip_first == srcip_last) {
 		srcip_offset = 0;
-		num_addrs = 1;
+		num_src_addrs = 1;
 	} else {
-		srcip_offset = rand() % (srcip_last - srcip_first);
-		num_addrs = ntohl(srcip_last) - ntohl(srcip_first) + 1;
+		uint32_t ip_first = ntohl(srcip_first);
+		uint32_t ip_last = ntohl(srcip_last);
+		assert(ip_first && ip_last);
+		assert(ip_last > ip_first);
+		uint32_t offset = (uint32_t) (aesrand_getword() & 0xFFFFFFFF);
+		srcip_offset = offset % (srcip_last - srcip_first);
+		num_src_addrs = ip_last - ip_first + 1;
 	}
 
 	// process the source port range that ZMap is allowed to use
 	num_src_ports = zconf.source_port_last - zconf.source_port_first + 1;
 	log_debug("send", "will send from %i address%s on %u source ports",
-		 num_addrs, ((num_addrs==1)?"":"es"), num_src_ports);
+		  num_src_addrs, ((num_src_addrs ==1 ) ? "":"es"),
+		  num_src_ports);
 
 	// global initialization for send module
 	assert(zconf.probe_module);
@@ -121,6 +139,14 @@ int send_init(void)
 						zconf.bandwidth, zconf.rate);
 	}
 
+	// Get the source hardware address, and give it to the probe
+	// module
+	if (get_iface_hw_addr(zconf.iface, zconf.hw_mac)) {
+		log_fatal("send", "could not retrieve hardware address for "
+			  "interface: %s", zconf.iface);
+		return NULL;
+	}
+
 	if (zconf.dryrun) {
 		log_info("send", "dryrun mode -- won't actually send packets");
 	}
@@ -129,17 +155,16 @@ int send_init(void)
 	validate_init();
 
 	zsend.start = now();	
-	return EXIT_SUCCESS;
+	return it;
 }
 
-int get_socket(void)
+static inline ipaddr_n_t get_src_ip(ipaddr_n_t dst, int local_offset)
 {
-	int sock = socket(AF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
-	if (sock <= 0) {
-		log_fatal("send", "couldn't create socket. "
-			  "Are you root? Error: %s\n", strerror(errno));
+	if (srcip_first == srcip_last) {
+		return srcip_first;
 	}
-	return sock;
+	return htonl(((ntohl(dst) + srcip_offset + local_offset) 
+			% num_src_addrs)) + srcip_first;
 }
 
 int get_dryrun_socket(void)
@@ -154,77 +179,49 @@ int get_dryrun_socket(void)
 			  "Error: %s\n", strerror(errno));
 	}
 	return sock;
-
-}
-
-static inline ipaddr_n_t get_src_ip(ipaddr_n_t dst, int local_offset)
-{
-	if (srcip_first == srcip_last) {
-		return srcip_first;
-	}
-	return htonl(((ntohl(dst) + srcip_offset + local_offset) 
-			% num_addrs)) + srcip_first;
 }
 
 // one sender thread
-int send_run(int sock)
+int send_run(int sock, shard_t *s)
 {
-	log_debug("send", "thread started");
+	log_trace("send", "send thread started");
 	pthread_mutex_lock(&send_mutex);
-	
-	struct sockaddr_ll sockaddr;
-	// get source interface index
-	struct ifreq if_idx;
-	memset(&if_idx, 0, sizeof(struct ifreq));
-	if (strlen(zconf.iface) >= IFNAMSIZ) {
-		log_error("send", "device interface name (%s) too long\n",
-				zconf.iface);
-		return -1;
-	}
-	strncpy(if_idx.ifr_name, zconf.iface, IFNAMSIZ-1);
-	if (ioctl(sock, SIOCGIFINDEX, &if_idx) < 0) {
-		perror("SIOCGIFINDEX");
-		return -1;
-	}
-	int ifindex = if_idx.ifr_ifindex;
-	// get source interface mac
-	struct ifreq if_mac;
-	memset(&if_mac, 0, sizeof(struct ifreq));
-	strncpy(if_mac.ifr_name, zconf.iface, IFNAMSIZ-1);
-	if (ioctl(sock, SIOCGIFHWADDR, &if_mac) < 0) {
-		perror("SIOCGIFHWADDR");
-		return -1;
-	}
-	// find source IP address associated with the dev from which we're sending.
-	// while we won't use this address for sending packets, we need the address
-	// to set certain socket options and it's easiest to just use the primary
-	// address the OS believes is associated.
-	struct ifreq if_ip;
-	memset(&if_ip, 0, sizeof(struct ifreq));
-	strncpy(if_ip.ifr_name, zconf.iface, IFNAMSIZ-1);
-	if (ioctl(sock, SIOCGIFADDR, &if_ip) < 0) {
-		perror("SIOCGIFADDR");
-		return -1;
-	}
-	// destination address for the socket
-	memset((void*) &sockaddr, 0, sizeof(struct sockaddr_ll));
-	sockaddr.sll_ifindex = ifindex;
-	sockaddr.sll_halen = ETH_ALEN;
-	memcpy(sockaddr.sll_addr, zconf.gw_mac, ETH_ALEN);
-
+	// Allocate a buffer to hold the outgoing packet
 	char buf[MAX_PACKET_SIZE];
 	memset(buf, 0, MAX_PACKET_SIZE);
-	zconf.probe_module->thread_initialize(buf, 
-					(unsigned char *)if_mac.ifr_hwaddr.sa_data, 
-					zconf.gw_mac, zconf.target_port);	
-	pthread_mutex_unlock(&send_mutex);
 
+	// OS specific per-thread init
+	if (send_run_init(sock)) {
+		return -1;
+	}
+
+	// MAC address length in characters
+	char mac_buf[(ETHER_ADDR_LEN * 2) + (ETHER_ADDR_LEN - 1) + 1];
+	char *p = mac_buf;
+	for(int i=0; i < ETHER_ADDR_LEN; i++) {
+		if (i == ETHER_ADDR_LEN-1) {
+			snprintf(p, 3, "%.2x", zconf.hw_mac[i]);
+			p += 2;
+		} else {
+			snprintf(p, 4, "%.2x:", zconf.hw_mac[i]);
+			p += 3;
+		}
+	}
+	log_debug("send", "source MAC address %s",
+			mac_buf);
+	if (zconf.probe_module->thread_initialize) {
+		zconf.probe_module->thread_initialize(buf, zconf.hw_mac, zconf.gw_mac,
+					      zconf.target_port);
+	}
+	pthread_mutex_unlock(&send_mutex);
+	
 	// adaptive timing to hit target rate
 	uint32_t count = 0;
 	uint32_t last_count = count;
 	double last_time = now();
 	uint32_t delay = 0;
 	int interval = 0;
+	uint32_t max_targets = s->state.max_targets;
 	volatile int vi;
 	if (zconf.rate > 0) {
 		// estimate initial rate
@@ -235,6 +232,7 @@ int send_run(int sock)
 		interval = (zconf.rate / zconf.senders) / 20;
 		last_time = now();
 	}
+	uint32_t curr = shard_get_cur_ip(s);
 	while (1) {
 		// adaptive timing delay
 		if (delay > 0) {
@@ -251,32 +249,23 @@ int send_run(int sock)
 				last_time = t;
 			}
 		}
-		// generate next ip from cyclic group and update global state
-		// (everything locked happens here)
-		pthread_mutex_lock(&send_mutex);
-		if (zsend.complete) {
-			pthread_mutex_unlock(&send_mutex);
+		if (zrecv.complete) {
+			s->cb(s->id, s->arg);
 			break;
 		}
-		if (zsend.sent >= zconf.max_targets) {
-			zsend.complete = 1;
-			zsend.finish = now();
-			pthread_mutex_unlock(&send_mutex);
+		if (s->state.sent >= max_targets) {
+			s->cb(s->id, s->arg);
 			break;
 		}
 		if (zconf.max_runtime && zconf.max_runtime <= now() - zsend.start) {
-			zsend.complete = 1;
-			zsend.finish = now();
-			pthread_mutex_unlock(&send_mutex);
+			s->cb(s->id, s->arg);
 			break;
 		}
-		uint32_t curr = cyclic_get_next_ip();
-		if (curr == zsend.first_scanned) {
-			zsend.complete = 1;
-			zsend.finish = now();
+		if (curr == 0) {
+			s->cb(s->id, s->arg);
+			break;
 		}
-		zsend.sent++;
-		pthread_mutex_unlock(&send_mutex);
+		s->state.sent++;
 		for (int i=0; i < zconf.packet_streams; i++) {
 			uint32_t src_ip = get_src_ip(curr, i);
 
@@ -285,26 +274,30 @@ int send_run(int sock)
 			zconf.probe_module->make_packet(buf, src_ip, curr, validation, i);
 
 			if (zconf.dryrun) {
+				pthread_mutex_lock(&send_mutex);
 				zconf.probe_module->print_packet(stdout, buf);
+				pthread_mutex_unlock(&send_mutex);
 			} else {
-					int l = zconf.probe_module->packet_length;
-					int rc = sendto(sock, buf + zconf.send_ip_pkts*sizeof(struct ethhdr),
-							l, 0,
-							(struct sockaddr *)&sockaddr,
-							sizeof(struct sockaddr_ll));
-					if (rc < 0) {
-						struct in_addr addr;
-						addr.s_addr = curr;
-						log_debug("send", "sendto failed for %s. %s",
-								  inet_ntoa(addr), strerror(errno));
-						pthread_mutex_lock(&send_mutex);
-						zsend.sendto_failures++;
-						pthread_mutex_unlock(&send_mutex);
-					}
+				int length = zconf.probe_module->packet_length;
+				void *contents = buf + zconf.send_ip_pkts*sizeof(struct ether_header);
+				int rc = send_packet(sock, contents, length);
+				if (rc < 0) {
+					struct in_addr addr;
+					addr.s_addr = curr;
+					log_debug("send", "send_packet failed for %s. %s",
+							  inet_ntoa(addr), strerror(errno));
+					s->state.failures++;
+				}
 			}
 		}
+		curr = shard_get_next_ip(s);
 	}
-	log_debug("send", "thread finished");
+	if (zconf.dryrun) {
+		pthread_mutex_lock(&send_mutex);
+		fflush(stdout);
+		pthread_mutex_unlock(&send_mutex);
+	}
+	log_debug("send", "thread %hu finished", s->id);
 	return EXIT_SUCCESS;
 }
 
